# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: test_ref_message.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='test_ref_message.proto',
  package='',
  serialized_pb=_b('\n\x16test_ref_message.proto\"L\n\x07TestRef\x12\x11\n\ttimestamp\x18\x01 \x01(\r\x12\x0e\n\x06\x66ield1\x18\x02 \x01(\x03\x12\x0e\n\x06\x66ield2\x18\x03 \x01(\x01\x12\x0e\n\x06\x66ield3\x18\x04 \x01(\t')
)
_sym_db.RegisterFileDescriptor(DESCRIPTOR)




_TESTREF = _descriptor.Descriptor(
  name='TestRef',
  full_name='TestRef',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='timestamp', full_name='TestRef.timestamp', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='field1', full_name='TestRef.field1', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='field2', full_name='TestRef.field2', index=2,
      number=3, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='field3', full_name='TestRef.field3', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=26,
  serialized_end=102,
)

DESCRIPTOR.message_types_by_name['TestRef'] = _TESTREF

TestRef = _reflection.GeneratedProtocolMessageType('TestRef', (_message.Message,), dict(
  DESCRIPTOR = _TESTREF,
  __module__ = 'test_ref_message_pb2'
  # @@protoc_insertion_point(class_scope:TestRef)
  ))
_sym_db.RegisterMessage(TestRef)


# @@protoc_insertion_point(module_scope)
